// File: MainDlg.cpp
//
// Description: Main application dialog demonstrating virtual printer
// transfer mode related configuration

#include "StdAfx.h"
#include "Configuration.h"
#include "SHDialogs.h"
#include "MainDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//----------------------------------------------------------------------------
// CMainDlg implementation
//----------------------------------------------------------------------------

CMainDlg::CMainDlg(CWnd* pParent /*= NULL*/) : CDialog(CMainDlg::IDD, pParent)
{
	// Load main icon
	m_hIcon = AfxGetApp()->LoadIcon(IDI_MAINICON);
}

//----------------------------------------------------------------------------

void CMainDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CLIENTAPPEDIT, m_ClientAppEdit);
	DDX_Control(pDX, IDC_WNDCLASSEDIT, m_WndClassEdit);
	DDX_Control(pDX, IDC_WNDTITLEEDIT, m_WndTitleEdit);
	DDX_Control(pDX, IDC_CLIPBOARDFMTEDIT, m_ClipboardFmtEdit);
	DDX_Control(pDX, IDC_MSGIDEDIT, m_MessageIdEdit);
	DDX_Control(pDX, IDC_TRANSFERMODECOMBO, m_TransferModeCombo);
	DDX_Control(pDX, IDC_OUTPUTPATHEDIT, m_OutputPathEdit);
	DDX_Control(pDX, IDC_EMFCHECK, m_EmfCheckBox);
	DDX_Control(pDX, IDC_BMPCHECK, m_BmpCheckBox);
	DDX_Control(pDX, IDC_PNGCHECK, m_PngCheckBox);
	DDX_Control(pDX, IDC_JPEGCHECK, m_JpegCheckBox);
	DDX_Control(pDX, IDC_TIFFCHECK, m_TiffCheckBox);
	DDX_Control(pDX, IDC_PDFCHECK, m_PdfCheckBox);
	DDX_Control(pDX, IDC_TXTCHECK, m_TxtCheckBox);
	DDX_Control(pDX, IDC_REDIRECTCHECK, m_RedirectCheckBox);
	DDX_Control(pDX, IDC_LANGUAGECHECK, m_LanguageCheckBox);
}

//----------------------------------------------------------------------------
// CMainDlg message handlers
//----------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CMainDlg, CDialog)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_EXEBROWSEBUTTON, &CMainDlg::OnBnClickedExebrowsebutton)
	ON_BN_CLICKED(IDC_DIRBROWSEBUTTON, &CMainDlg::OnBnClickedDirbrowsebutton)
	ON_BN_CLICKED(IDC_SAVEBUTTON, &CMainDlg::OnBnClickedSavebutton)
END_MESSAGE_MAP()

BOOL CMainDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon (big & small) for the dialog
	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon, FALSE);

	// Initialize "Transfer mode" combobox
	m_TransferModeCombo.AddString("Through the command line");
	m_TransferModeCombo.AddString("Through the WM_COPYDATA message");
	m_TransferModeCombo.AddString("Thtough the clipboard");

	// Load settings from the registry
	if (!m_PrnSettings.Load())
		AfxMessageBox("Can not load printer settings from the registry",
			MB_ICONERROR | MB_OK);

	// Set up the controls
	CString str;
	m_ClientAppEdit.SetWindowText(m_PrnSettings.m_sClientAppPath);
	m_WndClassEdit.SetWindowText(m_PrnSettings.m_sClientWndClass);
	m_WndTitleEdit.SetWindowText(m_PrnSettings.m_sClientWndTitle);
	m_ClipboardFmtEdit.SetWindowText(m_PrnSettings.m_sClipboardFormat);
	str.Format("%u (0x%.8X)", m_PrnSettings.m_iMessageId, m_PrnSettings.m_iMessageId);
	m_MessageIdEdit.SetWindowText(str);
	m_TransferModeCombo.SetCurSel((int) m_PrnSettings.m_TransferMode);
	m_OutputPathEdit.SetWindowText(m_PrnSettings.m_sOutputPath);

	m_EmfCheckBox.SetCheck(!m_PrnSettings.m_fHideEmfPage);
	m_BmpCheckBox.SetCheck(!m_PrnSettings.m_fHideBmpPage);
	m_PngCheckBox.SetCheck(!m_PrnSettings.m_fHidePngPage);
	m_JpegCheckBox.SetCheck(!m_PrnSettings.m_fHideJpegPage);
	m_TiffCheckBox.SetCheck(!m_PrnSettings.m_fHideTiffPage);
	m_PdfCheckBox.SetCheck(!m_PrnSettings.m_fHidePdfPage);
	m_TxtCheckBox.SetCheck(!m_PrnSettings.m_fHideTxtPage);
	m_RedirectCheckBox.SetCheck(!m_PrnSettings.m_fHideRedirectPage);
	m_LanguageCheckBox.SetCheck(!m_PrnSettings.m_fHideLanguagePage);

	return TRUE;
}

//----------------------------------------------------------------------------

void CMainDlg::OnPaint()
{
/*	This handler only draws the icon for minimized window
*/
	if (IsIconic())
	{
		// Device context for painting
		CPaintDC dc(this);

		// Erase background
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1)/2;
		int y = (rect.Height() - cyIcon + 1)/2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
		CDialog::OnPaint();
}

//----------------------------------------------------------------------------

void CMainDlg::OnBnClickedExebrowsebutton()
{
	char szFileName[MAX_PATH];
	if (OpenFileDialogBox(GetSafeHwnd(), szFileName, sizeof(szFileName)))
		m_ClientAppEdit.SetWindowText(szFileName);
}

//----------------------------------------------------------------------------

void CMainDlg::OnBnClickedDirbrowsebutton()
{
	char szFolderName[MAX_PATH];
	if (OpenFolderDialogBox(GetSafeHwnd(), szFolderName))
		m_OutputPathEdit.SetWindowText(szFolderName);
}

//----------------------------------------------------------------------------

#define ERROR_MSG "Message ID for the selected Transfer mode\r\n\
must be in range from 32768 to 49151"

void CMainDlg::OnBnClickedSavebutton()
{
	CString str;
	ULONG dwMsgId;

	// Get Message ID
	char* pStop = " (";
	m_MessageIdEdit.GetWindowText(str);
	dwMsgId = strtoul(str, &pStop, 10);
	str.Format("%u (0x%.8X)", dwMsgId, dwMsgId);
	m_MessageIdEdit.SetWindowText(str);

	// Check Message ID
	if ( (m_TransferModeCombo.GetCurSel() == ThroughClipboard) &&
		((dwMsgId < 0x8000) || (dwMsgId > 0xBFFF)) )
	{
		AfxMessageBox(ERROR_MSG, MB_ICONWARNING | MB_OK);
		return;
	}

	// Update settings in CPrnSettings object
	m_ClientAppEdit.GetWindowText(m_PrnSettings.m_sClientAppPath);
	m_WndClassEdit.GetWindowText(m_PrnSettings.m_sClientWndClass);
	m_WndTitleEdit.GetWindowText(m_PrnSettings.m_sClientWndTitle);
	m_ClipboardFmtEdit.GetWindowText(m_PrnSettings.m_sClipboardFormat);
	m_PrnSettings.m_iMessageId = dwMsgId;
	m_PrnSettings.m_TransferMode = (TransferModes) m_TransferModeCombo.GetCurSel();
	m_OutputPathEdit.GetWindowText(m_PrnSettings.m_sOutputPath);

	m_PrnSettings.m_fHideEmfPage = (m_EmfCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHideBmpPage = (m_BmpCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHidePngPage = (m_PngCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHideJpegPage = (m_JpegCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHideTiffPage = (m_TiffCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHidePdfPage = (m_PdfCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHideTxtPage = (m_TxtCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHideRedirectPage = (m_RedirectCheckBox.GetCheck() == BST_UNCHECKED);
	m_PrnSettings.m_fHideLanguagePage = (m_LanguageCheckBox.GetCheck() == BST_UNCHECKED);

	// Save settings to the registry
	if (m_PrnSettings.Save())
		EndDialog(0);
	else
		AfxMessageBox("Can not save printer settings to the registry",
			MB_ICONERROR | MB_OK);
}
