// File: SHDialogs.cpp
//
// Description: Special dialog functionality
// (.exe-file and folder selection dialogs)

#include "StdAfx.h"

//----------------------------------------------------------------------------

BOOL OpenFileDialogBox(HWND hDlg, LPTSTR lpszFileName, int iFileNameLen)
{
/*	This function opens OpenFile dialog and returns full path to the
	selected filename in lpszFileName parameter; function will return
	FALSE if file selection will be cancelled
*/
	OPENFILENAME par;
	memset(&par, 0, sizeof(OPENFILENAME));
	par.lStructSize = sizeof(OPENFILENAME);
	par.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY; 
	par.lpstrFilter = ".exe files (*.exe)\0*.exe\0All files (*.*)\0*.*\0";
	par.lpstrFile = lpszFileName; 
	par.nMaxFile = iFileNameLen; 
	par.lpstrDefExt = "exe"; 
	par.hwndOwner = hDlg;

	*lpszFileName = 0;
	return GetOpenFileName(&par);
}

//----------------------------------------------------------------------------

static HRESULT SHGetUIObjectFromFullPIDL(LPCITEMIDLIST pidl, HWND hwnd,
										 REFIID riid, void **ppv)
{
/*	This function retrieves the UIObject interface for the specified full PIDL
*/
	LPCITEMIDLIST pidlChild;
	IShellFolder* psf;
	*ppv = NULL;

	HRESULT hr = SHBindToParent(pidl, IID_IShellFolder, (void**) &psf, &pidlChild);
	if (SUCCEEDED(hr))
	{
		hr = psf->GetUIObjectOf(hwnd, 1, &pidlChild, riid, NULL, ppv);
		psf->Release();
	}
	return hr;
}

//----------------------------------------------------------------------------

static HRESULT SHILClone(LPCITEMIDLIST pidl, LPITEMIDLIST *ppidl)
{
	DWORD cbTotal = 0;

	if (pidl)
	{
		LPCITEMIDLIST pidl_temp = pidl;
		cbTotal += sizeof(pidl_temp->mkid.cb);
		while (pidl_temp->mkid.cb)
		{
			cbTotal += pidl_temp->mkid.cb;
			pidl_temp = (LPITEMIDLIST) (((BYTE*) pidl_temp) + pidl_temp->mkid.cb);
		}
	}
	*ppidl = (LPITEMIDLIST) CoTaskMemAlloc(cbTotal);
	if (*ppidl) CopyMemory(*ppidl, pidl, cbTotal);

	return *ppidl ? S_OK : E_OUTOFMEMORY;
}

//----------------------------------------------------------------------------

static HRESULT SHGetTargetFolderIDList(LPCITEMIDLIST pidlFolder,
									   LPITEMIDLIST* ppidl)
{
/*	This function gets the target PIDL for a folder PIDL. This deals with cases
	where a folder is an alias to a real folder, folder shortcuts, etc.
*/
	IShellLink *psl;
	*ppidl = NULL;

	HRESULT hr = SHGetUIObjectFromFullPIDL(pidlFolder, NULL,
		IID_IShellLink, (void**) &psl);

	if (SUCCEEDED(hr))
	{
		hr = psl->GetIDList(ppidl);
		psl->Release();
	}
	else
		// It's not a folder shortcut so get the PIDL normally
		hr = SHILClone(pidlFolder, ppidl);

    return hr;
}

//----------------------------------------------------------------------------

BOOL OpenFolderDialogBox(HWND hDlg, LPTSTR lpszFolderName)
{
/*	This function opens OpenFolder dialog and returns the selected folder;
	lpszFolderName must contain an address of a buffer to receive the file
	system path, this buffer must be at least MAX_PATH characters in size
*/
	BROWSEINFO par;
	memset(&par, 0, sizeof(BROWSEINFO));
	par.lpszTitle = "Select new folder for generated files:";
	par.ulFlags = BIF_DONTGOBELOWDOMAIN | BIF_RETURNONLYFSDIRS;
	par.hwndOwner = hDlg;

	*lpszFolderName = 0;
	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	LPITEMIDLIST pidl = SHBrowseForFolder(&par);

	if (pidl)
	{
		LPITEMIDLIST pidlTarget;
		if (SUCCEEDED(SHGetTargetFolderIDList(pidl, &pidlTarget)))
		{
			SHGetPathFromIDList(pidlTarget, lpszFolderName);
			CoTaskMemFree(pidlTarget);
		}
		CoTaskMemFree(pidl);
	}

	return (pidl != NULL);
}
