// File: Settings.cpp
//
// Description: Implementation of the CPrnSettings class
// containing printer preferences

#include "StdAfx.h"
#include "Settings.h"

//----------------------------------------------------------------------------

CPrnSettings::CPrnSettings()
{
/*	Constructor, sets the default values (settings)
*/
	LoadDefaults();
}

//----------------------------------------------------------------------------

void CPrnSettings::LoadDefaults()
{
/*	This function sets all parameters to default values without saving them
*/
	m_sClientAppPath = "X:\\YourFolder\\YourClient.exe";
	m_sClientWndClass.Empty();
	m_sClientWndTitle = "Sample Client Application";
	m_sClipboardFormat = "Your Virtual Printer Example clipboard format";
	m_iMessageId = 0x8000;
	m_TransferMode = ThroughCommandLine;
	m_sOutputPath.Empty();

	m_fHideEmfPage = m_fHideBmpPage = m_fHidePngPage = m_fHideJpegPage = false;
	m_fHideTiffPage = m_fHidePdfPage = m_fHideTxtPage = false;
	m_fHideRedirectPage = m_fHideLanguagePage = false;
}

//----------------------------------------------------------------------------

#define READ_STR(name, var, len) { \
	RegQueryValueEx(hKey, name, NULL, NULL, \
		(BYTE*) var.GetBuffer(len), &(dwSize = len)); \
	var.ReleaseBuffer(); }

#define READ_DWORD(name, var) { \
	RegQueryValueEx(hKey, name, NULL, NULL, \
		(BYTE*) &var, &(dwSize = sizeof(DWORD))); }

bool CPrnSettings::Load()
{
/*	This function loads printer settings from the registry
*/
	HKEY hKey = NULL;
	DWORD dwSize = 0;
	CString sKeyName;

	// Open the registry key
	sKeyName.Format("Software\\%s", REGISTRY_KEY);
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, sKeyName, 0, KEY_READ, &hKey) != ERROR_SUCCESS)
		return false;
	else
	{
		// Reset all settings to defaults
		LoadDefaults();

		// Read transfer mode related settings
		READ_STR("Client Application", m_sClientAppPath, MAX_PATH);
		READ_STR("Target Window Class", m_sClientWndClass, 256);
		READ_STR("Target Window Title", m_sClientWndTitle, 256);
		READ_STR("Clipboard Format", m_sClipboardFormat, 256);
		READ_STR("Output Directory", m_sOutputPath, MAX_PATH);
		READ_DWORD("Message ID", m_iMessageId);
		READ_DWORD("Transfer Mode", m_TransferMode);

		// Load property pages visibility settings
		READ_DWORD("EMF: Hide page", m_fHideEmfPage);
		READ_DWORD("BMP: Hide page", m_fHideBmpPage);
		READ_DWORD("PNG: Hide page", m_fHidePngPage);
		READ_DWORD("JPEG: Hide page", m_fHideJpegPage);
		READ_DWORD("TIFF: Hide page", m_fHideTiffPage);
		READ_DWORD("PDF: Hide page", m_fHidePdfPage);
		READ_DWORD("TXT: Hide page", m_fHideTxtPage);
		READ_DWORD("REDIRECT: Hide page", m_fHideRedirectPage);
		READ_DWORD("LANGUAGE: Hide page", m_fHideLanguagePage);

		RegCloseKey(hKey);
	}

	return true;
}

//----------------------------------------------------------------------------

#define WRITE_STR(name, var) { \
	lResult |= RegSetValueEx(hKey, name, 0, REG_SZ, (BYTE*) (LPCTSTR) var, \
		sizeof(TCHAR)*(var.GetLength() + 1)); }

#define WRITE_DWORD(name, var) { \
	lResult |= RegSetValueEx(hKey, name, 0, REG_DWORD, \
		(BYTE*) &var, sizeof(DWORD)); }

// Save settings to the registry
bool CPrnSettings::Save()
{
/*	This function saves all settings to the registry
*/
	HKEY hKey = NULL;
	DWORD dwSize = 0;
	CString sKeyName;

	// Open the registry key
	sKeyName.Format("Software\\%s", REGISTRY_KEY);
	LONG lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, sKeyName, 0, KEY_WRITE, &hKey);

	if (lResult == ERROR_SUCCESS)
	{
		// Write transfer mode related settings
		WRITE_STR("Client Application", m_sClientAppPath);
		WRITE_STR("Target Window Class", m_sClientWndClass);
		WRITE_STR("Target Window Title", m_sClientWndTitle);
		WRITE_STR("Clipboard Format", m_sClipboardFormat);
		WRITE_STR("Output Directory", m_sOutputPath);
		WRITE_DWORD("Message ID", m_iMessageId);
		WRITE_DWORD("Transfer Mode", m_TransferMode);

		// Load property pages visibility settings
		WRITE_DWORD("EMF: Hide page", m_fHideEmfPage);
		WRITE_DWORD("BMP: Hide page", m_fHideBmpPage);
		WRITE_DWORD("PNG: Hide page", m_fHidePngPage);
		WRITE_DWORD("JPEG: Hide page", m_fHideJpegPage);
		WRITE_DWORD("TIFF: Hide page", m_fHideTiffPage);
		WRITE_DWORD("PDF: Hide page", m_fHidePdfPage);
		WRITE_DWORD("TXT: Hide page", m_fHideTxtPage);
		WRITE_DWORD("REDIRECT: Hide page", m_fHideRedirectPage);
		WRITE_DWORD("LANGUAGE: Hide page", m_fHideLanguagePage);

		RegCloseKey(hKey);
	}

	return (lResult == ERROR_SUCCESS);
}
