// File: Settings.h
//
// Description: Interface for the CPrnSettings class
// containing printer preferences

#if !defined(__SETTINGS_H__)
#define __SETTINGS_H__

#pragma once

// The registry key (HKLM) where settings are stored
#define REGISTRY_KEY	"Your Virtual Printer Example"

// Print job information transfer modes
enum TransferModes {
    ThroughCommandLine	= 0,	// Through the command line
    ThroughCopydata		= 1,	// Through the WM_COPYDATA message
    ThroughClipboard	= 2		// Thtough the clipboard
};

// CPrnSettings class
class CPrnSettings
{
public:
	// Transfer mode related settings
	CString			m_sClientAppPath;		// Full path to the client application
	CString			m_sClientWndClass;		// Class name of the client application window
	CString			m_sClientWndTitle;		// Title of the client application window
	CString			m_sClipboardFormat;		// Name of the clipboard format
	int				m_iMessageId;			// Numeric value for message identification
	TransferModes	m_TransferMode;			// Print job information transfer mode
	CString			m_sOutputPath;			// Output directory for genereated files

	// Property pages visibility settings
	BOOL			m_fHideEmfPage;			// TRUE if "Emf" page is hidden
	BOOL			m_fHideBmpPage;			// TRUE if "Bmp" page is hidden
	BOOL			m_fHidePngPage;			// TRUE if "Png" page is hidden
	BOOL			m_fHideJpegPage;		// TRUE if "Jpeg" page is hidden
	BOOL			m_fHideTiffPage;		// TRUE if "Tiff" page is hidden
	BOOL			m_fHidePdfPage;			// TRUE if "Pdf" page is hidden
	BOOL			m_fHideTxtPage;			// TRUE if "Txt" page is hidden
	BOOL			m_fHideRedirectPage;	// TRUE if "Redirect" page is hidden
	BOOL			m_fHideLanguagePage;	// TRUE if "Language" page is hidden

	// Constructor, sets the default values (settings)
	CPrnSettings();

	// Load settings from the registry
	bool Load();

	// Save settings to the registry
	bool Save();

private:
	// Set all parameters to defaults (without saving them)
	void LoadDefaults();
};

#endif // !defined(__SETTINGS_H__)
